<?php
/**
 * This is the entrypoint for the locrMAPS for WooCommerce WordPress plugin.
 *
 * @package locrMAPS for WooCommerce
 */

declare(strict_types=1);

namespace Locr\WordPress\Plugin\LocrMapsForWooCommerce\Admin;

use Locr\WordPress\Plugin\LocrMapsForWooCommerce\DDD\Services\Image;
use Locr\WordPress\Plugin\LocrMapsForWooCommerce\Plugin;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * This class is responsible for the custom product.
 */
final class CustomProduct {
	/**
	 * This function initializes the custom product.
	 */
	public static function init(): void {
		add_action( 'wp_ajax_add_custom_product_to_cart', array( self::class, 'add_custom_product_to_cart' ) );
		add_action( 'wp_ajax_nopriv_add_custom_product_to_cart', array( self::class, 'add_custom_product_to_cart' ) );
		add_action( 'wp_ajax_download_custom_product_image', array( self::class, 'download_custom_product_image' ) );
		add_filter( 'kses_allowed_protocols', array( self::class, 'add_data_as_allowed_protocol' ), 10, 1 );
		add_action( 'woocommerce_thankyou', array( self::class, 'after_order_completed' ), 10, 1 );
		add_filter( 'woocommerce_admin_order_item_thumbnail', array( self::class, 'display_custom_product_thumbnail' ), 10, 3 );
		add_action( 'woocommerce_checkout_create_order_line_item', array( self::class, 'add_custom_product_image_filename' ), 10, 4 );
		add_filter( 'woocommerce_order_item_get_formatted_meta_data', array( self::class, 'modify_formatted_meta_data_for_custom_image_data' ), 10, 2 );
		add_action( 'woocommerce_remove_cart_item', array( self::class, 'remove_custom_product_image_filename' ), 10, 2 );
		add_action( 'woocommerce_before_delete_order', array( self::class, 'remove_custom_product_image_filename_before_delete_order_item' ), 10, 2 );
	}

	/**
	 * This function adds the data protocol to the allowed protocols.
	 *
	 * @param  array<mixed> $protocols The protocols.
	 * @return array<mixed>
	 */
	public static function add_data_as_allowed_protocol( array $protocols ): array {
		if ( is_admin() && ! in_array( needle: 'data', haystack: $protocols, strict: true ) ) {
			$protocols[] = 'data';
		}

		return $protocols;
	}

	/**
	 * This function adds the custom product image data.
	 *
	 * @param \WC_Order_Item_Product $item The item.
	 * @param mixed                  $cart_item_key The cart item key.
	 * @param array<mixed>           $values The values.
	 * @param mixed                  $order The order.
	 */
	public static function add_custom_product_image_filename( $item, $cart_item_key, $values, $order ): void {
		if ( isset( $values['custom_image_filename'] ) && is_string( $values['custom_image_filename'] ) ) {
			$item->add_meta_data( 'custom_image_filename', $values['custom_image_filename'], true );
		}
	}

	/**
	 * This function adds the custom product to the cart.
	 */
	public static function add_custom_product_to_cart(): void {
		$product_id         = intval( filter_input( INPUT_POST, 'product_id', FILTER_VALIDATE_INT, FILTER_NULL_ON_FAILURE ) ?? 0 );
		$quantity           = intval( filter_input( INPUT_POST, 'quantity', FILTER_VALIDATE_INT, FILTER_NULL_ON_FAILURE ) ?? 1 );
		$quantity           = max( 1, $quantity );
		$variation_id       = intval( filter_input( INPUT_POST, 'variation_id', FILTER_VALIDATE_INT, FILTER_NULL_ON_FAILURE ) ?? 0 );
		$product_variations = filter_input( INPUT_POST, 'product_variations', FILTER_DEFAULT, FILTER_NULL_ON_FAILURE );
		$custom_image_data  = filter_input( INPUT_POST, 'custom_image_data', FILTER_DEFAULT, FILTER_NULL_ON_FAILURE );

		$cart_item_data = array();
		if ( ! is_null( $custom_image_data ) && false !== $custom_image_data ) {
			$image      = Image::from_data_url( $custom_image_data );
			$upload_dir = Plugin::get_upload_folder();
			wp_mkdir_p( $upload_dir );
			$upload_filename = uniqid( md5( $image->data ) . '_', true ) . '.' . $image->type->to_file_extension();

			$cart_item_data['custom_image_filename'] = $upload_filename;

			global $wp_filesystem;
			require_once ABSPATH . 'wp-admin/includes/file.php'; // @phpstan-ignore-line
			WP_Filesystem();
			// @phpstan-ignore-next-line
			if ( ! $wp_filesystem->put_contents( $upload_dir . DIRECTORY_SEPARATOR . $upload_filename, $image->data ) ) {
				wp_send_json_error();
			}
		}

		$variation = array();
		if ( ! is_null( $product_variations ) && false !== $product_variations ) {
			$product_variations_decoded = json_decode( $product_variations, true );
			if ( is_array( $product_variations_decoded ) ) {
				foreach ( $product_variations_decoded as $value ) {
					if (
						is_array( $value ) &&
						isset( $value['variation_id'] ) &&
						$value['variation_id'] === $variation_id &&
						isset( $value['attributes'] ) &&
						is_array( $value['attributes'] )
					) {
						$variation = $value['attributes'];
						break;
					}
				}
			}
		}

		$cart_item_key = \WC()->cart->add_to_cart( $product_id, $quantity, $variation_id, $variation, $cart_item_data );

		if ( $cart_item_key ) {
			wp_send_json_success();
		} else {
			wp_send_json_error(
				array(
					'product_id'     => $product_id,
					'quantity'       => $quantity,
					'variation_id'   => $variation_id,
					'variation'      => $variation,
					'cart_item_data' => $cart_item_data,
				)
			);
		}
	}

	/**
	 * This function is called after the order is completed.
	 *
	 * @param  mixed $order_id The order ID.
	 * @return void
	 */
	public static function after_order_completed( mixed $order_id ): void {
		// do nothing.
	}

	/**
	 * This function displays the custom product thumbnail.
	 *
	 * @param string                 $product_image The product image.
	 * @param int                    $item_id The item ID.
	 * @param \WC_Order_Item_Product $item The item.
	 */
	public static function display_custom_product_thumbnail( $product_image, $item_id, $item ): string {
		$checked_disabled = '';
		$options          = get_option( Plugin::SLUG . '_options' );
		if ( is_array( $options ) && isset( $options[ Plugin::SLUG . '_preview_in_wc_order_disabled' ] ) && is_string( $options[ Plugin::SLUG . '_preview_in_wc_order_disabled' ] ) ) {
			$checked_disabled = checked( 'on', $options[ Plugin::SLUG . '_preview_in_wc_order_disabled' ], false );
		}
		if ( ! empty( $checked_disabled ) ) {
			return '';
		}

		$custom_image_filename = $item->get_meta( 'custom_image_filename', true );
		if ( is_string( $custom_image_filename ) && ! empty( $custom_image_filename ) ) {
			$image         = Image::from_file( Plugin::get_upload_folder() . DIRECTORY_SEPARATOR . $custom_image_filename );
			$resized_image = $image->resize_by_max_length( 100 );
			return '<img src="' . esc_attr( $resized_image->to_data_url() ) . '" alt="preview" title="preview" style="' . esc_attr( 'max-width: 100px; max-height: 100px;' ) . '"/>';
		}

		return $product_image;
	}

	/**
	 * This function downloads the custom product image.
	 */
	public static function download_custom_product_image(): void {
		if ( ! is_admin() ) {
			wp_die( 'Forbidden.', 'Forbidden.', array( 'response' => 403 ) );
		}

		$item_id = intval( filter_input( INPUT_GET, 'item_id', FILTER_VALIDATE_INT, FILTER_NULL_ON_FAILURE ) ?? 0 );

		check_admin_referer( 'download-custom-product-image_' . $item_id );

		$custom_image_filename = wc_get_order_item_meta( $item_id, 'custom_image_filename', true );
		if ( ! is_string( $custom_image_filename ) ) {
			wp_die( 'No custom product image found.', 'No custom product image found.', array( 'response' => 404 ) );
		}
		$full_filename = Plugin::get_upload_folder() . DIRECTORY_SEPARATOR . $custom_image_filename;
		if ( file_exists( $full_filename ) ) {
			$image              = Image::from_file( $full_filename );
			$filename_extension = $image->type->to_file_extension();
			$filename           = "custom_product_image_{$item_id}.{$filename_extension}";

			header( 'Content-Type: application/octet-stream' );
			header( 'Content-Disposition: attachment; filename="' . $filename . '"' );
			header( 'Content-Length: ' . strlen( $image->data ) );
			print $image->data; // phpcs:ignore
			die();
		} else {
			wp_die( 'No custom product image found.', 'No custom product image found.', array( 'response' => 404 ) );
		}
	}

	/**
	 * This function modifies the formatted meta data for the custom image data.
	 *
	 * @param array<mixed> $formatted_meta The formatted meta data.
	 * @param object       $that The that.
	 * @return array<mixed>
	 */
	public static function modify_formatted_meta_data_for_custom_image_data( $formatted_meta, $that ): array {
		foreach ( $formatted_meta as $key => $meta ) {
			if ( ! ( $meta instanceof \stdClass ) || 'custom_image_filename' !== $meta->key ) {
				continue;
			}

			$meta->display_key   = 'Preview';
			$meta->display_value = 'n.v.';

			if ( is_admin() ) {
				$item_id              = 0;
				$download_link_params = array( 'action' => 'download_custom_product_image' );
				if ( method_exists( $that, 'get_id' ) ) {
					$download_link_params['item_id'] = $that->get_id();
					if ( is_numeric( $that->get_id() ) ) {
						$item_id = intval( $that->get_id() );
					}
				}
				$download_link = admin_url( 'admin-ajax.php' ) . '?' . http_build_query( $download_link_params );
				if ( 0 !== $item_id ) {
					$download_link = wp_nonce_url( $download_link, 'download-custom-product-image_' . $item_id );
				}
				$meta->display_key   = 'Download';
				$meta->display_value = '<a href="' . esc_attr( $download_link ) . '" target="_blank">Link</a>';
			}
			break;
		}

		return $formatted_meta;
	}

	/**
	 * This function removes the custom product image filename.
	 *
	 * @param string   $cart_item_key The cart item key.
	 * @param \WC_Cart $that The that.
	 */
	public static function remove_custom_product_image_filename( $cart_item_key, $that ): void {
		$cart_item = $that->get_cart_item( $cart_item_key );
		if ( isset( $cart_item['custom_image_filename'] ) && is_string( $cart_item['custom_image_filename'] ) ) {
			$full_filename = Plugin::get_upload_folder() . DIRECTORY_SEPARATOR . $cart_item['custom_image_filename'];
			if ( file_exists( $full_filename ) ) {
				wp_delete_file( $full_filename );
			}
		}
	}

	/**
	 * This function removes the custom product image filename before delete order.
	 *
	 * @param int       $order_id The item ID.
	 * @param \WC_Order $order The order.
	 */
	public static function remove_custom_product_image_filename_before_delete_order_item( $order_id, $order ): void {
		$items = $order->get_items();
		foreach ( $items as $item ) {
			$custom_image_filename = $item->get_meta( 'custom_image_filename', true );
			if ( is_string( $custom_image_filename ) && ! empty( $custom_image_filename ) ) {
				$full_filename = Plugin::get_upload_folder() . DIRECTORY_SEPARATOR . $custom_image_filename;
				if ( file_exists( $full_filename ) ) {
					wp_delete_file( $full_filename );
				}
			}
		}
	}
}
